#include <stdio.h>
#include <string.h>
#include "lmp.h"
#include "lmpinput.h"

/* LMP main function
 *
 * It parses command line arguments and calls the L math
 * processor.
 * returns 0 if successfull.
 */
int main (argc, argv)
int argc;     /* Number of arguments */
char ** argv; /* Vector of arguments */
{
  lmp_value_type result;
  int i, base = -1, lmp_offset = 0, sig_prec=-1, prec=-1, parse_args=0, lmp_argc;
  char * endptr[1];

#ifdef DEBUG
  fprintf (stderr, "(DBG) main: %d arguments.\n", argc - 1);
#endif
  if (argc == 1)
    lmp_no_args ();

  if (!strcmp (argv[1], "--help"))
#ifdef DEBUG
  {
    fprintf (stderr, "(DBG) main: Echo help.\n");
#endif
    lmp_print_help ();
#ifdef DEBUG
  }
#endif

  if (!strcmp (argv[1], "--version"))
#ifdef DEBUG
  {
    fprintf (stderr, "(DBG) main: Echo version.\n");
#endif
    lmp_print_version ();
#ifdef DEBUG
  }
#endif

  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-pa") || !strcmp (argv[i], "--parse-args")) {
      parse_args=1;
      lmp_offset++;
    }
    else if (!strcmp (argv[i], "-b")) {
      if (i == argc-1) {
	fprintf (stderr, "No idea what you want, exiting.\n");
	exit (0);
      }
      lmp_offset += 2;
      base = STRTOVAL (argv[++i], endptr, 10);
      if (*endptr != 0 && base == 0) {
	fprintf (stderr, "Base argument must be a number, treating as 10.\n");
	base = 10;
      }
    }
    else if (!strncmp (argv[i], "--base=", 7)) {
      lmp_offset++;
      base = STRTOVAL (argv[i]+7, endptr, 10);
      if (*endptr != 0 && base == 0) {
	fprintf  (stderr, "Base argument must be a number, treating as 10.\n");
	base = 10;
      }
    }
    else if (!strcmp (argv[i], "-sp")) {
      if (i == argc-1) {
	fprintf (stderr, "No idea what you want, exiting.\n");
	exit (0);
      }
      lmp_offset += 2;
      sig_prec = STRTOVAL (argv[++i], endptr, 10);
      if (*endptr != 0 && sig_prec == 0) {
	fprintf (stderr, "Significant precision argument must be a number, treating as 6.\n");
	sig_prec = 6;
      }
    }
    else if (!strncmp (argv[i], "--significance=", 15)) {
      lmp_offset++;
      sig_prec = STRTOVAL (argv[i]+15, endptr, 10);
      if (*endptr != 0 && sig_prec == 0) {
	fprintf  (stderr, "Significant precision argument must be a number, treating as 6.\n");
	sig_prec = 6;
      }
    }
    else if (!strcmp (argv[i], "-p")) {
      if (i == argc-1) {
	fprintf (stderr, "No idea what you want, exiting.\n");
	exit (0);
      }
      lmp_offset += 2;
      prec = STRTOVAL (argv[++i], endptr, 10);
      if (*endptr != 0 && prec == 0) {
	fprintf (stderr, "Precision argument must be a number, treating as 6.\n");
	prec = 6;
      }
    }
    else if (!strncmp (argv[i], "--precision=", 12)) {
      lmp_offset++;
      prec = STRTOVAL (argv[i]+12, endptr, 10);
      if (*endptr != 0 && prec == 0) {
	fprintf  (stderr, "Precision argument must be a number, treating as 6.\n");
	prec = 6;
      }
    }
    else if (!strcmp (argv[i], "-vt")) {
      if (i == argc-1) {
	fprintf (stderr, "No idea what you want, exiting.\n");
	exit (0);
      }
      lmp_offset += 2;
      init_var_table (argv[++i]);
    }
    else if (!strncmp (argv[i], "--vartable=", 11)) {
      lmp_offset++;
      init_var_table (&argv[i][11]);
    }
  }

  /* Two methods are available:
   * - Parse each argument
   * - Take arguments as a parsed string
   */
  if (parse_args) { // Parse each argument, new method
    for (i = lmp_offset + 1; i < argc; i++) {
      int j;

      lmp_args = lmp_parse_string_by_elements (argv[i], &lmp_argc);
      lmp_argnum = 1;
      result = lmp_eval ();
      if (lmp_get_arg_num() != lmp_argnum)
#ifdef DEBUG
	{
	  fprintf (stderr, "(DBG) main: Evaluating finished, but still arguments left.\n");
#endif
	  lmp_error ("Nothing expected at argument %d\n", lmp_offset);
#ifdef DEBUG
	}
#endif
      if (base == -1 && sig_prec == -1 && prec == -1)
	printf ("%Lf\n", result);
      else if (base >= 2 && base <= 36)
	printf ("%s\n", base2base (base, result, prec));
      else if (sig_prec == -1 && prec == -1)
	fprintf (stderr, "Illegal base specification: %d\n", base);
      else
	printf (dyn_format ("", prec, sig_prec, "\n"), result);
    }
  }
  else { // Arguments are parsed already
    lmp_args = &argv [lmp_offset+1];
    result = lmp_eval ();
    if (lmp_get_arg_num() != argc - lmp_offset)
#ifdef DEBUG
      {
	fprintf (stderr, "(DBG) main: Evaluating finished, but still arguments left.\n");
#endif
	lmp_error ("Nothing expected at argument %d\n", lmp_offset);
#ifdef DEBUG
      }
#endif
    if (base == -1 && sig_prec == -1 && prec == -1)
      printf ("%Lf\n", result);
    else if (base >= 2 && base <= 36)
      printf ("%s\n", base2base (base, result, prec));
    else if (sig_prec == -1 && prec == -1)
      fprintf (stderr, "Illegal base specification: %d\n", base);
    else
      printf (dyn_format ("", prec, sig_prec, "\n"), result);
  }

  return 0;
}
