#include "lmp.h"
#ifdef DEBUG
#include <stdio.h>
#endif
#include "lmpmath.h"

/* Binary function processor
 *
 * This will compute all math functions with two operands.
 * returns the result of the operation
 */
lmp_value_type lmp_binary_proc (op, left, right)
lmp_operator op;      /* operation */
lmp_value_type left;  /* left operand */
lmp_value_type right; /* right operand */
{
  switch (op) {
  case LMP_OUTER_OR:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf or %Lf\n", left, right);
#endif
    return left || right;

  case LMP_OUTER_AND:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf and %Lf\n", left, right);
#endif
    return left && right;

  case LMP_EQUALS:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf == %Lf\n", left, right);
#endif
    return left == right;

  case LMP_UNEQUALS:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf != %Lf\n", left, right);
#endif
    return left != right;

  case LMP_GREATER_EQUAL:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf >= %Lf\n", left, right);
#endif
    return left >= right;

  case LMP_LOWER_EQUAL:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf <= %Lf\n", left, right);
#endif
    return left <= right;

  case LMP_GREATER:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf > %Lf\n", left, right);
#endif
    return left > right;

  case LMP_LOWER:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf < %Lf\n", left, right);
#endif
    return left < right;

  case LMP_ADDITION:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf + %Lf\n", left, right);
#endif
    return left + right;

  case LMP_SUBSTRACTION:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf - %Lf\n", left, right);
#endif
    return left - right;

  case LMP_MULTIPLICATION:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf * %Lf\n", left, right);
#endif
    return left * right;

  case LMP_DIVISION:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf / %Lf\n", left, right);
#endif
    return left / right;

  case LMP_MODULO:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Ld MOD %Ld\n", (long long) left, (long long) right);
#endif
    return MOD (left, right);

  case LMP_DIV_NO_REM:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Ld DIV %Ld\n", (long long) left, (long long) right);
#endif
    return (long long) left / (long long) right;

  case LMP_POWER:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: %Lf ^ %Lf\n", left, right);
#endif
    return POW (left, right);

  case LMP_LOG:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: log_%Lf %Lf\n", left, right);
#endif
    return LOGN (left) / LOGN (right);

  default:
#ifdef DEBUG
    fprintf (stderr, "(DBG) binary_proc: unknown operator\n");
#endif
    lmp_proc_error = LMP_UNKNOWN_OPERATOR;
  }

  return 0;
}

/* Unary function processor
 *
 * This will compute all math functions with one operand.
 * returns the result of the operation
 */
lmp_value_type lmp_unary_proc (op, operand)
lmp_operator op;         /* operation */
lmp_value_type operand;  /* operand */
{
  switch (op) {
  case LMP_SINE:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: sin %Lf\n", operand);
#endif
    return SIN (operand);

  case LMP_COSINE:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: cos %Lf\n", operand);
#endif
    return COS (operand);

  case LMP_TANGENT:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: tan %Lf\n", operand);
#endif
    return TAN (operand);

  case LMP_COTANGENT:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: cot %Lf\n", operand);
#endif
    return 1.0l / TAN (operand);

  case LMP_ARC_SINE:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: arcsin %Lf\n", operand);
#endif
    return ARCSIN (operand);

  case LMP_ARC_COSINE:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: arccos %Lf\n", operand);
#endif
    return ARCCOS (operand);

  case LMP_ARC_TANGENT:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: arctan %Lf\n", operand);
#endif
    return ARCTAN (operand);

  case LMP_LOGARITHM:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: ln %Lf\n", operand);
#endif
    return LOGN (operand);

  case LMP_LOG_10:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: lg %Lf\n", operand);
#endif
    return LOG10 (operand);

  case LMP_EXP:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: exp %Lf\n", operand);
#endif
    return EXP (operand);

  case LMP_SQRT:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: exp %Lf\n", operand);
#endif
    return SQRT (operand);

  default:
#ifdef DEBUG
    fprintf (stderr, "(DBG) unary_proc: unknown operator\n");
#endif
    lmp_proc_error = LMP_UNKNOWN_OPERATOR;
  }

  return 0;
}
